@@ -40,6 +40,8 @@ class _MM_SESSION_SPACE(obj.CType):
         one session.
         """
         for p in self.ProcessList.list_of_type("_EPROCESS", "SessionProcessLinks"):
+            if not p.is_valid():
+                break
             yield p
 
     @property
@@ -380,7 +382,7 @@ class tagWINDOWSTATION(obj.CType, windows.ExecutiveObjectMixin):
     def desktops(self):
         """A generator that yields the window station's desktops"""
         desk = self.rpdeskList.dereference()
-        while desk.is_valid() and desk.v() != 0:
+        while desk.is_valid() and desk.v() != 0 and desk.Name:
             yield desk
             desk = desk.rpdeskNext.dereference()
 
@@ -462,9 +464,9 @@ class tagDESKTOP(tagWINDOWSTATION):
         wins = []
         cur = win
         while cur.is_valid() and cur.v() != 0:
-            if cur in seen:
+            if cur.obj_offset in seen:
                 break
-            seen.add(cur)
+            seen.add(cur.obj_offset)
             wins.append(cur)
             cur = cur.spwndNext.dereference()
         while wins:
@@ -475,8 +477,11 @@ class tagDESKTOP(tagWINDOWSTATION):
             yield cur, level
 
             if cur.spwndChild.is_valid() and cur.spwndChild.v() != 0:
-                for info in self.windows(cur.spwndChild, filter = filter, level = level + 1):
-                    yield info
+                for xwin, xlevel in self.windows(cur.spwndChild, filter = filter, level = level + 1):
+                    if xwin.obj_offset in seen:
+                        break
+                    yield xwin, xlevel
+                    seen.add(xwin.obj_offset)
 
     def heaps(self):
         """Generator for the desktop heaps"""
