@@ -33,23 +33,28 @@ class linux_mount(linux_common.AbstractLinuxCommand):
     """Gather mounted fs/devices"""
 
     def _parse_mnt(self, mnt, ns, fs_types):
+        ret = None
+
+        if not mnt.mnt_root.is_valid():
+            return ret
+
         dev_name = mnt.mnt_devname.dereference_as("String", length = linux_common.MAX_STRING_LENGTH)
-        
-        #if not dev_name.is_valid() or len(dev_name) < 2:
-        #    return
+       
+        if not dev_name.is_valid():
+            return ret
 
         fstype = mnt.mnt_sb.s_type.name.dereference_as("String", length = linux_common.MAX_STRING_LENGTH)
 
         if not fstype.is_valid():
-            return 
+            return ret
 
         if str(fstype) not in fs_types:
-            return 
+            return ret
 
         path = linux_common.do_get_path(mnt.mnt_sb.s_root, mnt.mnt_parent, mnt.mnt_root, mnt)
 
         if path == []:
-            return
+            return ret
 
         mnt_string = self._calc_mnt_string(mnt)
 
@@ -58,12 +63,12 @@ class linux_mount(linux_common.AbstractLinuxCommand):
         else:
             rr = "rw"
         
-        yield mnt.mnt_sb, dev_name, path, fstype, rr, mnt_string
+        return mnt.mnt_sb, str(dev_name), path, fstype, rr, mnt_string
 
     def calculate(self):
         linux_common.set_plugin_members(self)
         mntptr   = obj.Object("Pointer", offset = self.addr_space.profile.get_symbol("mount_hashtable"), vm = self.addr_space)
-        mnt_list = obj.Object(theType = "Array", offset = mntptr, vm = self.addr_space, targetType = "list_head", count = 4099)
+        mnt_list = obj.Object(theType = "Array", offset = mntptr, vm = self.addr_space, targetType = "list_head", count = 8200)
 
         if self.profile.has_type("mount"):
             mnttype = "mount"
@@ -72,40 +77,51 @@ class linux_mount(linux_common.AbstractLinuxCommand):
 
         ns = None        
         fs_types = self._get_filesystem_types()
-        seen = {}
-
-        all_mnts = []
 
+        hash_mnts = {}
         for outerlist in mnt_list:
             if outerlist == outerlist.next:
                 continue
 
             for mnt in outerlist.list_of_type(mnttype, "mnt_hash"):
-                all_mnts.append(mnt)
-                all_mnts.append(mnt.mnt_parent)
+                hash_mnts[mnt]            = 1
+                hash_mnts[mnt.mnt_parent] = 1
+                hash_mnts[mnt.mnt_parent.mnt_parent] = 1
 
-        tmp_mnts = []
-        for mnt in all_mnts:
+        child_mnts = {}
+        for mnt in hash_mnts:
             for child_mnt in mnt.mnt_child.list_of_type(mnttype, "mnt_child"):
-                tmp_mnts.append(child_mnt)
-                tmp_mnts.append(child_mnt.mnt_parent)
+                child_mnts[child_mnt]            = 1
+                child_mnts[child_mnt.mnt_parent] = 1
+                child_mnts[child_mnt.mnt_parent.mnt_parent] = 1
+        
+        all_mnts = list(set(hash_mnts.keys() + child_mnts.keys()))
     
-        all_mnts = all_mnts + tmp_mnts
-   
-        tmp_mnts = []
+        list_mnts = {}
         for mnt in all_mnts:
             for child_mnt in mnt.mnt_list.list_of_type(mnttype, "mnt_list"):
-                tmp_mnts.append(child_mnt)
-                tmp_mnts.append(child_mnt.mnt_parent)
+                list_mnts[child_mnt]            = 1
+                list_mnts[child_mnt.mnt_parent] = 1
+                list_mnts[child_mnt.mnt_parent.mnt_parent] = 1
        
-        all_mnts = all_mnts + tmp_mnts   
-
-        for mnt in all_mnts:
+        all_mnts = list(set(all_mnts + list_mnts.keys()))
+ 
+        seen = {}
+        for (idx, mnt) in enumerate(all_mnts):
             if mnt.mnt_sb.v() not in seen:
-                for (mnt_sb, dev_name, path, fstype, rr, mnt_string) in self._parse_mnt(mnt, ns, fs_types):
-                    yield (mnt_sb, dev_name, path, fstype, rr, mnt_string)
-            
-            seen[mnt.mnt_sb.v()] = 1
+                ret = self._parse_mnt(mnt, ns, fs_types)
+                        
+                mark = False
+                
+                if ret:
+                    (mnt_sb, dev_name, path, fstype, rr, mnt_string) = ret
+
+                    if not (dev_name == "devtmpfs" and path == "/"):
+                        yield (mnt_sb, dev_name, path, fstype, rr, mnt_string)
+                        mark = True
+
+                if mark:
+                    seen[mnt.mnt_sb.v()] = 1
 
     def _calc_mnt_string(self, mnt):
         ret = ""
