@@ -309,7 +309,10 @@ class IDT(common.AbstractWindowsCommand):
 
         for n, entry, addr, module in data:
 
-            if module:
+            if addr == 0:
+                module_name = "NOT USED"
+                sect_name = ''
+            elif module:
                 module_name = str(module.BaseDllName or '')
                 sect_name = self.get_section_name(module, addr)
             else:
