@@ -194,10 +194,7 @@ class PsXview(common.AbstractWindowsCommand, sessions.SessionsMixin):
         ps_sources['csrss'] = self.check_csrss_handles(all_tasks)
         ps_sources['pspcid'] = self.check_pspcid(addr_space)
         ps_sources['session'] = self.check_sessions(addr_space)
-        if addr_space.profile.metadata.get('major', 0) == 6 and addr_space.profile.metadata.get('minor', 0) >= 2:
-            ps_sources['deskthrd'] = {}
-        else:
-            ps_sources['deskthrd'] = self.check_desktop_thread(addr_space)
+        ps_sources['deskthrd'] = self.check_desktop_thread(addr_space)
 
         # Build a list of offsets from all sources
         seen_offsets = []
