@@ -1,14 +1,13 @@
 # Volatility
 # Copyright (C) 2008-2013 Volatility Foundation
-# Copyright (C) 2011 Jamie Levy (Gleeda) <jamie.levy@gmail.com>
+# Copyright (C) 2011 Jamie Levy (Gleeda) <jamie@memoryanalysis.net>
 #
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +21,7 @@
 """
 @author:       Jamie Levy (gleeda)
 @license:      GNU General Public License 2.0
-@contact:      jamie.levy@gmail.com
+@contact:      jamie@memoryanalysis.net
 @organization: Volatility Foundation
 """
 
@@ -33,8 +32,11 @@ import volatility.scan as scan
 import volatility.utils as utils
 import volatility.addrspace as addrspace
 import volatility.obj as obj
+import volatility.debug as debug
 import struct
 import binascii
+import os
+import volatility.poolscan as poolscan
 
 ATTRIBUTE_TYPE_ID = {
     0x10:"STANDARD_INFORMATION",
@@ -119,6 +121,12 @@ class MFT_FILE_RECORD(obj.CType):
         return ''.join([c for c in str if (ord(c) > 31 or ord(c) == 9) and ord(c) <= 126])
 
     def add_path(self, fileinfo):
+        # it doesn't really make sense to add regular files to parent directory,
+        # since they wouldn't actually be in the middle of a file path, but at the end
+        # therefore, we'll return for regular files
+        if not self.is_directory():
+            return
+        # otherwise keep a record of the directory that we've found
         cur = MFT_PATHS_FULL.get(int(self.RecordNumber), None)
         if cur == None or cur["filename"].find("~") != -1 and fileinfo.is_valid():
             temp = {}
@@ -127,30 +135,164 @@ class MFT_FILE_RECORD(obj.CType):
             MFT_PATHS_FULL[int(self.RecordNumber)] = temp
 
     def get_full_path(self, fileinfo):
+        if self.obj_vm._config.DEBUGOUT:
+            print "Building path for file {0}".format(fileinfo.get_name())
         parent = ""
-        parent_id = fileinfo.ParentDirectory & 0xffffff
         path = self.remove_unprintable(fileinfo.get_name()) or "(Null)"
+        try:
+            parent_id = fileinfo.ParentDirectory & 0xffffff
+        except struct.error:
+            return path
         if int(self.RecordNumber) == 5 or int(self.RecordNumber) == 0:
             return path
+        seen = set()
         while parent != {}:
+            seen.add(parent_id)
             parent = MFT_PATHS_FULL.get(int(parent_id), {})
             if parent == {} or parent["filename"] == "" or int(parent_id) == 0 or int(parent_id) == 5:
                 return path
-            path = parent["filename"] + "\\" + path
+            path = "{0}\\{1}".format(parent["filename"], path)
             parent_id = parent["ParentDirectory"] & 0xffffff
+            if parent_id in seen:
+                return path
         return path
 
+    def is_directory(self):
+        return int(self.Flags) & 0x2
+
+    def is_file(self):
+        return int(self.Flags) & 0x2 == 0
+
+    def is_inuse(self):
+        return int(self.Flags) & 0x1 == 0x1
+
     def get_mft_type(self):
-        thetype = "In Use & " if self.Flags & 0x1 == 0x1 else ""
-        if int(self.Flags) & 0x2:
-            thetype += "Directory"
-        elif int(self.Flags) & 0x2 == 0:
-            thetype += "File"
-        return thetype.rstrip(" & ")
+        return "{0}{1}".format("In Use & " if self.is_inuse() else "",
+               "Directory" if self.is_directory() else "File")
         
+    def parse_attributes(self, mft_buff, check = True, entrysize = 1024):
+        next_attr = self.ResidentAttributes
+        end = mft_buff.find("\xff\xff\xff\xff")
+        if end == -1:
+            end = entrysize
+        attributes = []
+        dataseen = False
+        while next_attr != None and next_attr.obj_offset <= end:
+            try:
+                attr = ATTRIBUTE_TYPE_ID.get(int(next_attr.Header.Type), None)
+            except struct.error:
+                next_attr = None
+                attr = None
+                continue
+            if attr == None:
+                next_attr = None
+            elif attr == "STANDARD_INFORMATION":
+                if self.obj_vm._config.DEBUGOUT:
+                    print "Found $SI"
+                if not check or next_attr.STDInfo.is_valid():
+                    attributes.append((attr, next_attr.STDInfo))
+                next_off = next_attr.STDInfo.obj_offset + next_attr.ContentSize
+                if next_off == next_attr.STDInfo.obj_offset:
+                    next_attr = None
+                    continue
+                next_attr = self.advance_one(next_off, mft_buff, end)
+            elif attr == 'FILE_NAME':
+                if self.obj_vm._config.DEBUGOUT:
+                    print "Found $FN"
+                self.add_path(next_attr.FileName)
+                if not check or next_attr.FileName.is_valid():
+                    attributes.append((attr, next_attr.FileName))
+                next_off = next_attr.FileName.obj_offset + next_attr.ContentSize
+                if next_off == next_attr.FileName.obj_offset:
+                    next_attr = None
+                    continue
+                next_attr = self.advance_one(next_off, mft_buff, end)
+            elif attr == "OBJECT_ID":
+                if self.obj_vm._config.DEBUGOUT:
+                    print "Found $ObjectId"
+                if next_attr.Header.NonResidentFlag == 1:
+                    attributes.append((attr, "Non-Resident"))
+                    next_attr = None
+                    continue
+                else:
+                    attributes.append((attr, next_attr.ObjectID))
+                next_off = next_attr.ObjectID.obj_offset + next_attr.ContentSize
+                if next_off == next_attr.ObjectID.obj_offset:
+                    next_attr = None
+                    continue
+                next_attr = self.advance_one(next_off, mft_buff, end)
+            elif attr == "DATA":
+                if self.obj_vm._config.DEBUGOUT:
+                    print "Found $DATA"
+                if next_attr.Header.NameOffset > 0 and next_attr.Header.NameLength > 0:
+                    adsname = ""
+                    if next_attr != None and next_attr.Header != None and next_attr.Header.NameOffset and next_attr.Header.NameLength:
+                        nameloc = next_attr.obj_offset + next_attr.Header.NameOffset
+                        adsname = obj.Object("NullString", vm = self.obj_vm, offset = nameloc, length = next_attr.Header.NameLength * 2)
+                        if adsname != None and adsname.strip() != "" and dataseen:
+                            attr += " ADS Name: {0}".format(adsname.strip())
+                dataseen = True
+                try:
+                    if next_attr.ContentSize == 0:
+                        next_off = next_attr.obj_offset + self.obj_vm.profile.get_obj_size("RESIDENT_ATTRIBUTE")
+                        next_attr = self.advance_one(next_off, mft_buff, end)
+                        attributes.append((attr, ""))
+                        continue
+                    start = next_attr.obj_offset + next_attr.ContentOffset
+                    theend = min(start + next_attr.ContentSize, end)
+                except struct.error:
+                    next_attr = None
+                    continue
+                if next_attr.Header.NonResidentFlag == 1:
+                    thedata = "" 
+                else:
+                    try:
+                        contents = mft_buff[start:theend]
+                    except TypeError:
+                        next_attr = None
+                        continue
+                    thedata = contents 
+                attributes.append((attr, thedata))
+                next_off = theend
+                if next_off == start:
+                    next_attr = None
+                    continue
+                next_attr = self.advance_one(next_off, mft_buff, end)
+            elif attr == "ATTRIBUTE_LIST":
+                if self.obj_vm._config.DEBUGOUT:
+                    print "Found $AttributeList"
+                if next_attr.Header.NonResidentFlag == 1:
+                    attributes.append((attr, "Non-Resident"))
+                    next_attr = None
+                    continue
+                next_attr.process_attr_list(self.obj_vm, self, attributes, check)
+                next_attr = None
+            else:
+                next_attr = None
+
+        return attributes
+
+    def advance_one(self, next_off, mft_buff, end):
+        item = None
+        attr = None
+        cursor = 0 
+
+        if next_off == None:
+            return None
+
+        while attr == None and cursor <= end:
+            try:
+                val = struct.unpack("<I", mft_buff[next_off + cursor: next_off + cursor + 4])[0]
+                attr = ATTRIBUTE_TYPE_ID.get(val, None)
+                item = obj.Object('RESIDENT_ATTRIBUTE', vm = self.obj_vm,
+                            offset = next_off + cursor)
+            except struct.error:
+                return None
+            cursor += 1
+        return item
 
 class RESIDENT_ATTRIBUTE(obj.CType):
-    def process_attr_list(self, bufferas, mft_entry, attributes = [], check = False):
+    def process_attr_list(self, bufferas, mft_entry, attributes = [], check = True):
         start = 0
         end = self.obj_offset + self.ContentSize
         while start < end:
@@ -171,7 +313,7 @@ class RESIDENT_ATTRIBUTE(obj.CType):
                         attributes.append(("FILE_NAME (AL)", theitem))
             except struct.error:
                 return
-            if item.Length == 0:
+            if item.Length <= 0:
                 return
             start += item.Length
 
@@ -179,8 +321,24 @@ class STANDARD_INFORMATION(obj.CType):
     # XXX need a better check than this
     # we return valid if we have _any_ timestamp other than Null
     def is_valid(self):
-        return obj.CType.is_valid(self) and (self.ModifiedTime.v() != 0 or self.MFTAlteredTime.v() != 0 or \
-                self.FileAccessedTime.v() != 0 or self.CreationTime.v() != 0) 
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0 
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0 
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0 
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
+        return obj.CType.is_valid(self) and (modified != 0 or mftaltered != 0 or \
+                accessed != 0 or creation != 0)
 
     def get_type_short(self):
         if self.Flags == None:
@@ -224,18 +382,19 @@ class STANDARD_INFORMATION(obj.CType):
             self.get_type())
 
     def body(self, path, record_num, size, offset):
-        if path == "":
+        if path.strip() == "" or path == None:
             # if the path is null we just try to get the filename 
             # from our dictionary and print the body file output
             record = MFT_PATHS_FULL.get(int(record_num), {}) 
+            path = "(Possible non-base entry, extra $SI or invalid $FN)"
             if record != {}:
                 # we include with the found filename a note that this may be a 
                 # non-base entry.  the analyst can investigate these types of records
                 # on his/her own by comparing record numbers in output or examining the 
                 # given physical offset in memory for example
-                path = record["filename"] + " (Possible non-base entry, extra $SI or invalid $FN)"
+                path = "{0} {1}".format(record["filename"], path)
 
-        return "[MFT STD_INFO] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
+        return "[{9}MFT STD_INFO] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
             path,
             offset,
             record_num,
@@ -244,7 +403,8 @@ class STANDARD_INFORMATION(obj.CType):
             self.FileAccessedTime.v(),
             self.ModifiedTime.v(),
             self.MFTAlteredTime.v(),
-            self.CreationTime.v())
+            self.CreationTime.v(),
+            self.obj_vm._config.MACHINE)
 
 class FILE_NAME(STANDARD_INFORMATION):
     def remove_unprintable(self, str):
@@ -254,8 +414,24 @@ class FILE_NAME(STANDARD_INFORMATION):
     # we return valid if we have _any_ timestamp other than Null
     # filename must also be a non-empty string
     def is_valid(self):
-        return obj.CType.is_valid(self) and (self.ModifiedTime.v() != 0 or self.MFTAlteredTime.v() != 0 or \
-                self.FileAccessedTime.v() != 0 or self.CreationTime.v() != 0) and \
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
+        return obj.CType.is_valid(self) and (modified != 0 or mftaltered != 0 or \
+                accessed != 0 or creation != 0) and \
                 self.remove_unprintable(self.get_name()) != ""
 
     def get_name(self):
@@ -279,14 +455,17 @@ class FILE_NAME(STANDARD_INFORMATION):
             self.remove_unprintable(self.get_name()))
 
     def get_full(self, full):
-        return "{0:20} {1:30} {2:30} {3:30} {4}".format(str(self.CreationTime),
-            str(self.ModifiedTime),
-            str(self.MFTAlteredTime),
-            str(self.FileAccessedTime),
-            self.remove_unprintable(full))
+        try:
+            return "{0:20} {1:30} {2:30} {3:30} {4}".format(str(self.CreationTime),
+                str(self.ModifiedTime),
+                str(self.MFTAlteredTime),
+                str(self.FileAccessedTime),
+                self.remove_unprintable(full))
+        except struct.error:
+            return None
 
     def body(self, path, record_num, size, offset):
-        return "[MFT FILE_NAME] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
+        return "[{9}MFT FILE_NAME] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
             path,
             offset,
             record_num,
@@ -295,7 +474,8 @@ class FILE_NAME(STANDARD_INFORMATION):
             self.FileAccessedTime.v(),
             self.ModifiedTime.v(),
             self.MFTAlteredTime.v(),
-            self.CreationTime.v())
+            self.CreationTime.v(),
+            self.obj_vm._config.MACHINE)
 
 class OBJECT_ID(obj.CType):
     # Modified from analyzeMFT.py:
@@ -383,7 +563,6 @@ MFT_types = {
                                         target = obj.Curry(obj.Object, "unsigned char")),
     }],
 
-
     'STANDARD_INFORMATION': [0x48, {
         'CreationTime': [0x0, ['WinTimeStamp', dict(is_utc = True)]],
         'ModifiedTime': [0x8, ['WinTimeStamp', dict(is_utc = True)]],
@@ -519,116 +698,70 @@ class MFTParser(common.AbstractWindowsCommand):
     """ Scans for and parses potential MFT entries """
     def __init__(self, config, *args, **kwargs):
         common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
-        config.add_option('CHECK', short_option = 'C', default = False,
-                          help = 'Only print entries w/o null timestamps',
+        config.add_option("OFFSET", short_option = "o", default = None, 
+                          help = "Physical offset for MFT Entries (comma delimited)")
+        config.add_option('NOCHECK', short_option = 'N', default = False,
+                          help = 'Only all entries including w/null timestamps',
                           action = "store_true")
-
         config.add_option("ENTRYSIZE", short_option = "E", default = 1024,
                           help = "MFT Entry Size",
                           action = "store", type = "int")
+        config.add_option('DUMP-DIR', short_option = 'D', default = None,
+                      cache_invalidator = False,
+                      help = 'Directory in which to dump extracted resident files')
+        config.add_option("MACHINE", default = "",
+                        help = "Machine name to add to timeline header")
+        config.add_option("DEBUGOUT", default = False,
+                        help = "Output debugging messages",
+                        action = "store_true")
+
     def calculate(self):
+        if self._config.MACHINE != "":
+            self._config.update("MACHINE", "{0} ".format(self._config.MACHINE))
+        offsets = []
         address_space = utils.load_as(self._config, astype = 'physical')
-        scanner = MFTScanner(needles = ['FILE', 'BAAD'])
-        mft_entries = []
-        print "Scanning for MFT entries and building directory, this can take a while"
-        for offset in scanner.scan(address_space):
-            mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
-            bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
-            mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas,
+        if self._config.OFFSET != None:
+            items = [int(o, 16) for o in self._config.OFFSET.split(',')]
+            for offset in items:
+                mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
+                bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
+                mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas, offset = 0)
+                offsets.append((offset, mft_entry))
+        else:
+            scanner = poolscan.MultiPoolScanner(needles = ['FILE', 'BAAD'])
+            print "Scanning for MFT entries and building directory, this can take a while"
+            seen = []
+            for _, offset in scanner.scan(address_space):
+                mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
+                bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
+                mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas,
                                offset = 0)
-            next_attr = mft_entry.ResidentAttributes
-            end = mft_buff.find("\xff\xff\xff\xff")
-            if end == -1:
-                end = self._config.ENTRYSIZE
-            attributes = []
-            while next_attr != None and next_attr.obj_offset <= end:
-                try:
-                    attr = ATTRIBUTE_TYPE_ID.get(int(next_attr.Header.Type), None)
-                except struct.error:
-                    next_attr = None
-                    attr = None
+                temp = mft_entry.advance_one(mft_entry.ResidentAttributes.STDInfo.obj_offset + mft_entry.ResidentAttributes.ContentSize, mft_buff, self._config.ENTRYSIZE)
+                name = ""
+                if temp != None:
+                    try:
+                        mft_entry.add_path(temp.FileName)
+                        name = temp.FileName.get_name()
+                    except struct.error:
+                        if self._config.DEBUGOUT:
+                            print "Problem entry at offset:", hex(offset)
+                        continue
+    
+                if (int(mft_entry.RecordNumber), name) in seen:
                     continue
-                if attr == None:
-                    next_attr = None
-                elif attr == "STANDARD_INFORMATION":
-                    if next_attr.STDInfo.is_valid() or not self._config.CHECK:
-                        attributes.append((attr, next_attr.STDInfo))
-                    next_off = next_attr.STDInfo.obj_offset + next_attr.ContentSize
-                    if next_off == next_attr.STDInfo.obj_offset:
-                        next_attr = None
-                        continue
-                    next_attr = self.advance_one(next_off, mft_buff, end)
-                elif attr == 'FILE_NAME':
-                    mft_entry.add_path(next_attr.FileName)
-                    if next_attr.FileName.is_valid() or not self._config.CHECK:
-                        attributes.append((attr, next_attr.FileName))
-                    next_off = next_attr.FileName.obj_offset + next_attr.ContentSize
-                    if next_off == next_attr.FileName.obj_offset:
-                        next_attr = None
-                        continue
-                    next_attr = self.advance_one(next_off, mft_buff, end)
-                elif attr == "OBJECT_ID":
-                    if next_attr.Header.NonResidentFlag == 1:
-                        attributes.append((attr, "Non-Resident"))
-                        next_attr = None
-                        continue
-                    else:
-                        attributes.append((attr, next_attr.ObjectID))
-                    next_off = next_attr.ObjectID.obj_offset + next_attr.ContentSize
-                    if next_off == next_attr.ObjectID.obj_offset:
-                        next_attr = None
-                        continue
-                    next_attr = self.advance_one(next_off, mft_buff, end)
-                elif attr == "DATA":
-                    start = next_attr.obj_offset + next_attr.ContentOffset
-                    theend = min(start + next_attr.ContentSize, end)
-                    if next_attr.Header.NonResidentFlag == 1:
-                        thedata = "Non-Resident"
-                    else:
-                        try:
-                            contents = mft_buff[start:theend]
-                        except TypeError:
-                            next_attr = None
-                            continue
-                        thedata = "\n".join(["{0:010x}: {1:<48}  {2}".format(o, h, ''.join(c)) for o, h, c in utils.Hexdump(contents)])
-                        if len(thedata) == 0:
-                            thedata = "(Empty)"
-                    attributes.append((attr, thedata))
-                    next_off = theend 
-                    if next_off == start: 
-                        next_attr = None
-                        continue
-                    next_attr = self.advance_one(next_off, mft_buff, end)
-                elif attr == "ATTRIBUTE_LIST":
-                    if next_attr.Header.NonResidentFlag == 1:
-                        attributes.append((attr, "Non-Resident"))
-                        next_attr = None
-                        continue
-                    next_attr.process_attr_list(bufferas, mft_entry, attributes, self._config.CHECK)
-                    next_attr = None
                 else:
-                    next_attr = None
-            mft_entries.append((offset, mft_entry, attributes))
+                    seen.append((int(mft_entry.RecordNumber), name))
+                offsets.append((offset, mft_entry, mft_buff))
 
-        return mft_entries
-
-
-    def advance_one(self, next_off, mft_buff, end):
-        item = None
-        attr = None
-        cursor = 0 
-        while attr == None and cursor <= end:
-            bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
-            item = obj.Object('RESIDENT_ATTRIBUTE', vm = bufferas,
-                                offset = next_off + cursor)
-            try:
-                attr = ATTRIBUTE_TYPE_ID.get(int(item.Header.Type), None)
-            except struct.error:
-                return item
-            cursor += 1
-        return item
+        for offset, mft_entry, mft_buff in offsets:
+            if self._config.DEBUGOUT:
+                print "Processing MFT Entry at offset:", hex(offset)
+            attributes = mft_entry.parse_attributes(mft_buff, not self._config.NOCHECK, self._config.ENTRYSIZE)
+            yield offset, mft_entry, attributes
 
     def render_body(self, outfd, data):
+        if self._config.DUMP_DIR != None and not os.path.isdir(self._config.DUMP_DIR):
+            debug.error(self._config.DUMP_DIR + " is not a directory")
         # Some notes: every base MFT entry should have one $SI and at lease one $FN
         # Usually $SI occurs before $FN
         # We'll make an effort to get the filename from $FN for $SI
@@ -636,42 +769,64 @@ class MFTParser(common.AbstractWindowsCommand):
         for offset, mft_entry, attributes in data:
             si = None
             full = ""
+            datanum = 0
             for a, i in attributes:
+                # we'll have a default file size of -1 for records missing $FN attributes
+                # note that file size found in $FN may not actually be accurate and will most likely
+                # be 0.  See Carrier, pg 363
+                size = -1
                 if a.startswith("STANDARD_INFORMATION"):
                     if full != "":
                         # if we are here, we've hit one $FN attribute for this entry already and have the full name
                         # so we can dump this $SI
-                        outfd.write("0|{0}\n".format(i.body(full, mft_entry.RecordNumber, int(mft_entry.EntryUsedSize), offset)))
+                        outfd.write("0|{0}\n".format(i.body(full, mft_entry.RecordNumber, size, offset)))
                     elif si != None:
                         # if we are here then we have more than one $SI attribute for this entry
                         # since we don't want to lose its info, we'll just dump it for now
                         # we won't have full path, but we'll have a filename most likely
-                        outfd.write("0|{0}\n".format(i.body("", mft_entry.RecordNumber, int(mft_entry.EntryUsedSize), offset)))
+                        outfd.write("0|{0}\n".format(i.body("", mft_entry.RecordNumber, size, offset)))
                     elif si == None:
                         # this is the usual case and we'll save the $SI to process after we get the full path from the $FN
                         si = i
                 elif a.startswith("FILE_NAME"):
                     if hasattr(i, "ParentDirectory"):
                         full = mft_entry.get_full_path(i)
-                        outfd.write("0|{0}\n".format(i.body(full, mft_entry.RecordNumber, int(mft_entry.EntryUsedSize), offset)))
+                        size = int(i.RealFileSize)
+                        outfd.write("0|{0}\n".format(i.body(full, mft_entry.RecordNumber, size, offset)))
                         if si != None:
-                            outfd.write("0|{0}\n".format(si.body(full, mft_entry.RecordNumber, int(mft_entry.EntryUsedSize), offset)))
+                            outfd.write("0|{0}\n".format(si.body(full, mft_entry.RecordNumber, size, offset)))
                             si = None
+                elif a.startswith("DATA"):
+                    if len(str(i)) > 0:
+                        file_string = ".".join(["file", "0x{0:x}".format(offset), "data{0}".format(datanum), "dmp"])
+                        datanum += 1
+                        if self._config.DUMP_DIR != None:
+                            of_path = os.path.join(self._config.DUMP_DIR, file_string)
+                            of = open(of_path, 'wb')
+                            of.write(i)
+                            of.close()
+
             if si != None:
                 # here we have a lone $SI in an MFT entry with no valid $FN.  This is most likely a non-base entry
-                outfd.write("0|{0}\n".format(si.body("", mft_entry.RecordNumber, int(mft_entry.EntryUsedSize), offset)))
+                outfd.write("0|{0}\n".format(si.body("", mft_entry.RecordNumber, -1, offset)))
 
     def render_text(self, outfd, data):
+        if self._config.DUMP_DIR != None and not os.path.isdir(self._config.DUMP_DIR):
+            debug.error(self._config.DUMP_DIR + " is not a directory")
         border = "*" * 75
         for offset, mft_entry, attributes in data:
             if len(attributes) == 0:
                 continue
-            outfd.write(border + "\n")
+            outfd.write("{0}\n".format(border))
             outfd.write("MFT entry found at offset 0x{0:x}\n".format(offset))
             outfd.write("Attribute: {0}\n".format(mft_entry.get_mft_type())) 
             outfd.write("Record Number: {0}\n".format(mft_entry.RecordNumber))
             outfd.write("Link count: {0}\n".format(mft_entry.LinkCount))
             outfd.write("\n")
+            # there can be more than one resident $DATA attribute
+            # e.g. ADS.  Therfore we need to differentiate somehow
+            # to avoid clobbering.  For now we'll use a counter (datanum)
+            datanum = 0
             for a, i in attributes:
                 if i == None:
                     outfd.write("${0}: malformed entry\n".format(a))
@@ -685,14 +840,25 @@ class MFTParser(common.AbstractWindowsCommand):
                     if hasattr(i, "ParentDirectory"):
                         full = mft_entry.get_full_path(i)
                         self.table_header(outfd, i.get_header())
-                        outfd.write("{0}\n".format(i.get_full(full)))
+                        output = i.get_full(full)
+                        if output == None:
+                            continue
+                        outfd.write("{0}\n".format(output))
                     else:
                         outfd.write("{0}\n".format(str(i)))
-                elif a == "DATA":
-                    outfd.write("\n$DATA\n")
-                    outfd.write("{0}\n".format(str(i)))
+                elif a.startswith("DATA"):
+                    outfd.write("\n${0}\n".format(a))
+                    contents = "\n".join(["{0:010x}: {1:<48}  {2}".format(o, h, ''.join(c)) for o, h, c in utils.Hexdump(i)])
+                    outfd.write("{0}\n".format(str(contents)))
+                    if len(str(i)) > 0:
+                        file_string = ".".join(["file", "0x{0:x}".format(offset), "data{0}".format(datanum), "dmp"])
+                        datanum += 1
+                        if self._config.DUMP_DIR != None:
+                            of_path = os.path.join(self._config.DUMP_DIR, file_string)
+                            of = open(of_path, 'wb')
+                            of.write(i)
+                            of.close()
                 elif a == "OBJECT_ID":
                     outfd.write("\n$OBJECT_ID\n")
                     outfd.write(str(i))
-            outfd.write("\n" + border + "\n") 
-
+            outfd.write("\n{0}\n".format(border))
